// Funktion: 
// Timer0 wird verwendet, um den 1-wire zu lesen/schreiben. Dazu wird er in verschiedenen Modi betrieben, um die verschiedenen Zeiten auszumessen. 
// INT0 erkennt jeweils eine Flanke am 1-wire Pin und startet dann entsprechend den Timer 0. 
// Die Adresse ergibt sich aus dem Familiycode (0x28 Thermometer) der Deviceid, die 5 mal wiederholt wird und einer variablen.   
// Die Variable kann 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40 und 0x80 sein um bertragungsfehler zu minimieren. 
// transdata[0:8] wird jeweis bertragen. dabei wird transdata[0] und [1] jeweils mit der der Adresse zugehrenden "Temperatur" 
// gefllt, die anderen werden fr alle Adressen identisch aufgefllt. 
// Eine zweite Deviceid_lausch kann fr eine Adresse eingegeben werden, auf der das Device nur zuhrt um den Busabzuhren

// Function: 
// Timer0 is used for timing of the 1-wire interface. It is used in different modes to measure the different timings. 
//INT0 is used to detect the edges on the 1-wire pin and starts then timer0 as required. 
// The adress results from the familycode 0x28 (Thermometer) and the deviceid which is repeated 5 times plus a variable which can be 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40 und 0x80.
// transdata[0:8] is transmitted with transdata[0] and transdata[1] being the temperature. 
// A second deviceid can be used to simply listen to the bus. In this mode the HR20 is a slave to another HR20 and follows its settings. 

 
// Pinning
// PD2/INT0:   1-Wire


#include <avr/io.h>
#include <avr/wdt.h>
#include <math.h>
#include <avr/interrupt.h>
#include "onewire.h"

//#define USE_TIMER0
volatile signed char scratchpad_read[2];           // Scratchpaddaten, which are transmitted fromother devices and that are being listend to. 

// Fuer One Wire Interface
// Byte 0: Temprature LSB
// Byte 1: Temprature MSB
// Byte 2: User Byte 1 EEPROM
// Byte 3: User Byte 2  EEPROM
// Byte 4: Configuration  EEPROM
// Byte 5: Reserved  (FFh)
// Byte 6: Reserved  (0Ch)
// Byte 7: Reserved (10h)
// Byte 8: CRC
// Definitionen fr den Zustandsanzeiger 


// Initialisierung der Hardware
void onewire_init()
{
/* Code for Mega8 und Mega32 */ 
// Clock wird ueber Fuses eingestellt:
// CKSEL = 0100;   Fuse Low Byte Bits 3:0
    onewire_cmd1 = 0xff;
    onewire_cmd2 = 0x0f;     // Controlbyte received over onewire
    counter2val = 0;         // erstmal auf 0 zur Einschaltverzoegerung
    counter2val_pre = 0;         // erstmal auf 0 zur Einschaltverzoegerung
    lauschmode = 0;         // Onewire in listening mode


    //DDRB = 0x0;          // Pin auf Eingang 
    Onewire_Pin_clear;
    Onewire_Pin_set_Input;


    // ATmega: Mode #4 für Timer1 und voller MCU-Takt (Prescale=1)
#ifdef USE_TIMER0
    TCCR0 = 0x03;    // Prescaler 1/64 fuer 1-wire 1 zaehlertakt entspricht 8 us
#else
    TCCR1A = 0x0;
    TCCR1B = 0x03;    // Prescaler 1/64
#endif

#ifdef __AVR_ATmega8__
    MCUCR |= (1 << ISC00);          // Interrupt auf beide Flanken
    GICR |= (1 << INT0) ;      // Externen Interrupt freigeben
#else
    PCMSK0 |= (1 << ONEWIREPIN);          // Interrupt auf Externen Pin PE2 
    EIMSK |= (1 << PCIE0) ;      // Externen Interrupt freigeben
#endif
    transdata[0] = 0;
    transdata[5] = 0xff;  // Initial Data in Scratchpad   
    transdata[6] = 0x0C;  //  Data in Scratchpad   
    transdata[7] = 0x10;  // Data in Scratchpad   
    scratchpad_read[0] = 0xff;
    status_global = IDLE;
//PORTB |= 0x01;


}




// Timer interrupt routine
#ifdef __AVR_ATtiny13__
ISR (TIM0_OVF_vect)
#elif defined (__AVR_ATmega8__)
#endif

#ifdef USE_TIMER0
ISR (TIMER0_OVF_vect)
#else 
ISR (TIMER1_OVF_vect)
#endif    // Port als Ausgang
{
    unsigned char tim0_i, status;
    signed int temperaturwert, temperaturwert_bruch;
#ifdef USE_TIMER0
    TIMSK0 &= ~(1 << TOIE0);       // Timer Interrupt off
#else
    TIMSK1 &= ~(1 << TOIE1);       // Timer Interrupt off
#endif
    status = status_global;
          Onewire_Pin_set_Input;
    if (status == RESET){
	Onewire_Pin_set_Output;
          status = PRESENCEPULSE;
          bitcount = 0;
#ifdef USE_TIMER0
          TCNT0 = ~PRESENCEZEIT;     // Start new for Presencepulse length
          TIFR0 |= (1 << TOV0);
          TIMSK0 |= (1 << TOIE0);       // Timer Interrupt on
#else
          TCNT1 = ~PRESENCEZEIT;     // Neu Starten zum bestimmen der Presencepulselaenge
          TIFR1 |= (1 << TOV1);
          TIMSK1 |= (1 << TOIE1);       // Timer Interrupt on

#endif
    }
    else if (status == PRESENCEPULSE){    // Presence Puls done, Timer INT off, Pin auf input, wait for Opcode 
          status = RECEIVE_OPCODE;
          bitcount = 0;     // TINT wird oben ausgeschaltet
    }
    else if (status & (RECEIVE_OPCODE | MATCHROM | WRITEMEM)){
	    bitcount++;
    	    if (bitcount == 1) {
		transbyte = 0;
	    }
            transbyte = transbyte >> 1;            // Aktuelles Byte received
            if (Onewire_Pin_check_One){
		transbyte |= 0x080;
	    } 
            if (bitcount == 8){
		if (status == RECEIVE_OPCODE) {
                        if (transbyte == 0x55){   // 0x55 -> Matchrom
                              status = MATCHROM;
                              transbyte = 0;    // New                        
	   		}
                     	else if (transbyte == 0x4E){
                              status = WRITEMEM;
                              transbyte = 0;
                     	}
                     	else if (transbyte == 0x48){  // Write data to EEPROM
                              status = WRITEMEM;
                              transbyte = 0;
                     	}
                     	else if (transbyte == 0x48){  // Write data to EEPROM at Adress 2 and 3 should be configurable
                            for (tim0_i = 2; tim0_i < 4; tim0_i++){
                                while(EECR & (1<<EEPE)) ;
				/* Set Programming mode */
				//EECR = (0<<EEPM1)|(0>>EEPM0);
				/* Set up address and data registers */
				EEARL = tim0_i;            // EEPROM Address
				EEDR = transdata[tim0_i];
				/* Write logical one to EEMPE */
				EECR |= (1<<EEMPE);
				/* Start eeprom write by setting EEPE */
				EECR |= (1<<EEPE);
			    }
			      status = IDLE;
                              transbyte = 0;
                     	}
                        else if (transbyte == 0xBE){    // read data 
                              if (lauschmode == 0) status = READMEM;
			      else status = WRITEMEM;
                              shift_reg = 0;     // Schiebergister für CRC loeschen
			     transbyte = transdata[0];
                        }
			else {
				status = IDLE;
			}
			bitcount = 0;
			bytecount = 0;
		}
		else if (status == MATCHROM){      // byte 0 is familycode, byte 1-5 are DEVICEID, byte 6 is adress           
         //uart_putc('1');
		if (bytecount == 0){
                          	if (transbyte == FAMILYCODE) ;
	        		else status = IDLE;
				lauschmode = 0;
                     	} 
       		      	else if (bytecount == 6){    // different virtuelle Adresses for each value, not required for rondostat 
       				if (transbyte == 1){
				       if (lauschmode == 1) lauschmode = 1;
                                       else {
					temperaturwert = temperatur[0];
                                       	temperaturwert_bruch = temperatur_bruch[0];
					}	
				}
       				else if (transbyte == 2){ 
				       if (lauschmode == 1) lauschmode = 2;	
                                       else {
					temperaturwert = temperatur[1];
                                       	temperaturwert_bruch = temperatur_bruch[1];
					}	
				}
       				else if (transbyte == 4){ 
				       if (lauschmode == 1) lauschmode = 4;	
                                       else {
					temperaturwert = temperatur[2];
                                       	temperaturwert_bruch = temperatur_bruch[2];
					}	
				}
       				else if (transbyte == 8){ 
				       if (lauschmode == 1) lauschmode = 8;	
				       else {
						temperaturwert = temperatur[3];
                                       		temperaturwert_bruch = temperatur_bruch[3];
				       }
				}
       				else if (transbyte == 16){ 
				       if (lauschmode == 1) lauschmode = 16;	
				       else {
						temperaturwert = temperatur[4];
                                       		temperaturwert_bruch = temperatur_bruch[4];
				       }
				}
       				else if (transbyte == 32){ 
				       if (lauschmode == 1) lauschmode = 32;	
				       else {
						temperaturwert = temperatur[5];
                                       		temperaturwert_bruch = temperatur_bruch[5];
				       }
				}
       				else if (transbyte == 64){ 
				       if (lauschmode == 1) lauschmode = 64;	
				       else {
						temperaturwert = temperatur[6];
                                       		temperaturwert_bruch = temperatur_bruch[6];
				       }
				}
       				else if (transbyte == 128){     // Temperatur im Wintergarten
                                       if (lauschmode == 1) lauschmode = 128;
				       else {
						temperaturwert = temperatur[7];
                                       		temperaturwert_bruch = temperatur_bruch[7];
				       }
				}
				else {
					 status = IDLE;         // Not for us, Telegram is for someone else
                                       temperaturwert = 127;
                                       temperaturwert_bruch = 0;
				}
                                transdata[0] = ((temperaturwert << 4) & 0x00f0) | temperaturwert_bruch; // Data in Scratchpad   
                                if (temperaturwert >= 0) transdata[1] = (temperaturwert >> 4) & 0x07;  // Data in Scratchpad   
                                else transdata[1] = (temperaturwert >> 4) | 0xf8;

		    	}
       		      	else if (bytecount < 6){
       				if (transbyte ==  DEVICEID) lauschmode = 0;
				else if (transbyte ==  DEVICEID_LAUSCH){
					 lauschmode = 1 ;
                                }
				else  status = IDLE;
		    	}
               	    	else {                  // Byte 8
				status = RECEIVE_OPCODE; // Actually we should chech the CRC here, but too lazy to implement
                                  //       uart_putc(lauschmode);
		      	}
               	    	if (status == IDLE){ 
		      		bytecount = 0;
               	    	}
                    	bytecount++;
		    	bitcount = 0;
              	}	
		else if (status == WRITEMEM){           // Could also be READMEM from another Controller            
         		if (lauschmode == 0) {          // Read data from bus
				if (bytecount == 0){       // Beim Schreiben werden zwei Bytes bertragen.
       					onewire_cmd1 = transbyte;
                          		//transdata[2] = transbyte;
                     		} 
       		      		else {
       					onewire_cmd2 = transbyte;
                          		//transdata[3] = transbyte;
					status = RECEIVE_OPCODE;
					bytecount = 0;
		    		}
			}
			else {                          // Wir lesen fuer/von andere bestimmte Daten vom Bus 
				if (bytecount == 1) {
					if (lauschmode == 1) temperatur_read[0] = transbyte;
					else if (lauschmode == 2) temperatur_read[1] = transbyte;
					else if (lauschmode == 4) temperatur_read[2] = transbyte;
					else if (lauschmode == 8) temperatur_read[3] = transbyte;
					else if (lauschmode == 0x10) temperatur_read[4] = transbyte;
					else if (lauschmode == 0x20) temperatur_read[5] = transbyte;
					else if (lauschmode == 0x40) temperatur_read[6] = transbyte;
					else if (lauschmode == 0x80) temperatur_read[7] = transbyte;
				}
				else if (bytecount == 5) {
					 scratchpad_read[0] = transbyte;
                             //   	uart_putc(transbyte);
				}
				else if (bytecount == 6) {
					scratchpad_read[1] = transbyte;
				}
				else if (bytecount == 7) scratchpad_read[2] = transbyte;
			}
                    	bytecount++;
		    	bitcount = 0;
              	}	
    	  }
    }
    else {                 // status == READMEM oder rest
            Onewire_Pin_set_Input;
    }
    status_global = status;
}



// Edge detection  on 1-Wire pin, 
//ISR (INT0_vect)
ONEWIRE_INT_ROUTINE
{
    unsigned char status;
    status = status_global;
    if (status != PRESENCEPULSE)  Onewire_Pin_set_Input;
//PINB |= 0x01; 
    if (Onewire_Pin_check_One_pine) {
#ifdef USE_TIMER0
            if ((TCNT0 < 0xE0) && (TCNT0 > 35)) {    // Reset pulse wenn Puls >30 und < F0 geaendert auf 35
                  TIFR0 |= (1 << TOV0);                   // Delete Timer Overflow 
                  TCNT0 = ~PRESENCEWAITZEIT;                  // Timer Neu Starten Fuer Presencepulse
                  TIMSK0 |= (1 << TOIE0);       // Timer Interrupt an
#else
            if ((TCNT1 < 0xE0) && (TCNT1 > 35)) {    // Reset pulse wenn Puls >30 und < F0 geaendert auf 35
//PINB |= 0x01; 
                 TIFR1 |= (1 << TOV1);                   // Timer Overflow loeschen
                  TCNT1 = ~PRESENCEWAITZEIT;                  // Timer Neu Starten Fuer Presencepulse
                  TIMSK1 |= (1 << TOIE1);       // Timer Interrupt an
#endif
            status = RESET;
            }
      }
      else {                                      // Fallende Flanke am One Wire
          if (status == READMEM){
                if ((transbyte & 0x01) == 0){
                        Onewire_Pin_set_Output;
                  }
#ifdef USE_TIMER0
                TIFR0 |= (1 << TOV0);         // TimerOverflow loeschen
                TCNT0  = ~SENDEZEIT;
                TIMSK0 |= (1 << TOIE0);       // Timer Interrupt an
#else
                TIFR1 |= (1 << TOV1);         // TimerOverflow loeschen
                TCNT1  = ~SENDEZEIT;
                TIMSK1 |= (1 << TOIE1);       // Timer Interrupt an
#endif
         	fb_bit = (transbyte ^ shift_reg) & 0x01;   // CRC berechnen
         	shift_reg = shift_reg >> 1;
         	if (fb_bit)
           		shift_reg = shift_reg ^ 0x8c;
                bitcount++;
                transbyte = transbyte >> 1;
                if (bitcount == 8){
                        bitcount = 0;
                        bytecount++;
                        if (bytecount == 8){
				 transbyte = shift_reg;  // CRC senden
                        }
                        else if (bytecount == 9) status = IDLE;  // CRC senden
			else transbyte = transdata[bytecount];
                }
          }
          else if (status == IDLE) {           // Erste fallende Flanke
#ifdef USE_TIMER0
              TIFR0 |= (1 << TOV0);         // TimerOverflow loeschen
              TIMSK0 &= ~(1 << TOIE0);       // Timer Interrupt aus
              TCNT0 = 0;
#else
              TIFR1 |= (1 << TOV1);         // TimerOverflow loeschen
              TIMSK1 &= ~(1 << TOIE1);       // Timer Interrupt aus
              TCNT1 = 1;
#endif
             // status = RESET;                // Erste fallende Flanke nach Idle, Timer nutzen um Lnge des Pulses zu messen
          }
          else if (status & (RECEIVE_OPCODE | MATCHROM | WRITEMEM)){
#ifdef USE_TIMER0
                TCNT0 = ~ABTASTZEIT;          // Zeichen abtasten in ABTASTZEIT
                TIFR0 |= (1 << TOV0);         // TimerOverflow loeschen
                TIMSK0 |= (1 << TOIE0);       // Timer Interrupt an
#else
                TCNT1 = ~ABTASTZEIT;          // Zeichen abtasten in ABTASTZEIT
                TIFR1 |= (1 << TOV1);         // TimerOverflow loeschen
                TIMSK1 |= (1 << TOIE1);       // Timer Interrupt an
#endif
          }
          else if (status == RESET) {           // Da hat ein anderer einen Presence Pulse gesendet, da haengen wir uns dran
          	status = PRESENCEPULSE;
          	bitcount = 0;
#ifdef USE_TIMER0
          	TCNT0 = ~PRESENCEZEIT;     // Neu Starten zum bestimmen der Presencepulselaenge
          	TIFR0 |= (1 << TOV0);       // Timer OVF loeschen
          	TIMSK0 |= (1 << TOIE0);       // Timer Interrupt an
#else
          	TCNT1 = ~PRESENCEZEIT;     // Neu Starten zum bestimmen der Presencepulselaenge
          	TIFR1 |= (1 << TOV1);       // Timer OVF loeschen
          	TIMSK1 |= (1 << TOIE1);       // Timer Interrupt an
#endif
          }
	  else if (status == PRESENCEPULSE);   // Gar nichts tun, Pegelnderung kommt vom eigenen Timer beim Senden oder anderen
	  else {
               status = IDLE;
#ifdef USE_TIMER0
                TCNT0 = 0;
                TIFR0 |= (1 << TOV0);         // TimerOverflow loeschen
                TIMSK0 &= ~(1 << TOIE0);       // Timer Interrupt aus
#else
                TCNT1 = 1;
                TIFR1 |= (1 << TOV1);         // TimerOverflow loeschen
                TIMSK1 &= ~(1 << TOIE1);       // Timer Interrupt aus
#endif
          }
     }
    status_global = status;
}

// Fallende Flanke
// Status=IDLE, TCNT->0, TINT->0   status -> RESET
// Steigende Flanke
// Status=RESET,   
