// Funktion: 
// Timer0 wird verwendet, um den 1-wire zu lesen/schreiben. Dazu wird er in verschiedenen Modi betrieben, um die verschiedenen Zeiten auszumessen. 
// INT0 erkennt jeweils eine Flanke am 1-wire Pin und startet dann entsprechend den Timer 0. 
// Die Adresse ergibt sich aus dem Familiycode (0x28 Thermometer) der Deviceid, die 5 mal wiederholt wird und einer variablen.   
// Die Variable kann 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40 und 0x80 sein um bertragungsfehler zu minimieren. 
// transdata[0:8] wird jeweis bertragen. dabei wird transdata[0] und [1] jeweils mit der der Adresse zugehrenden "Temperatur" 
// gefllt, die anderen werden fr alle Adressen identisch aufgefllt. 
// Eine zweite Deviceid_lausch kann fr eine Adresse eingegeben werden, auf der das Device nur zuhrt um den Busabzuhren

// Pinbelegung
// PD2/INT0:   1-Wire


//#include <avr/io.h>
//#include <avr/wdt.h>
//#include <math.h>
//#include <avr/interrupt.h>


#define F_CPU                 8000000
#include "config.h"     // General Setup configuration

// Prototypen der Funktionen
void onewire_init(void);

// Fuer One Wire Interface
// Byte 0: Temprature LSB
// Byte 1: Temprature MSB
// Byte 2: User Byte 1 EEPROM
// Byte 3: User Byte 2  EEPROM
// Byte 4: Configuration  EEPROM
// Byte 5: Reserved  (FFh)
// Byte 6: Reserved  (0Ch)
// Byte 7: Reserved (10h)
// Byte 8: CRC
// Definitionen fr den Zustandsanzeiger 
#define IDLE 0x00                   // Warten auf Signale auf 1-wire
#define RESET 0x01
#define PRESENCEPULSE 0x02
#define WAITOPCODE 0x04
#define READMEM 0x08
#define WRITEEEPROM 0x09
#define RECEIVE_OPCODE 0x10
#define MATCHROM 0x20
#define SEARCHROM 0x40
#define WRITEMEM 0x80


// definitionen fr die OneWire erkennung
#define FAMILYCODE 0x28       // Identifies as 18B20 Thermometer 0x28, 
#define DEVICEID_LAUSCH  0x54        // Deviceid for other HR20 on the bus we are only listening to 

#ifdef WOHNZIMMER
  #define DEVICEID  0x61        // Fers Wohnzimmer(Livingroom), 61   fuer den Wintergarten 58
  #warning "Compiliere fur Wohnzimmer"
#elif defined WINTERGARTEN
  #define DEVICEID  0x58     // Um Speicher zu sparen wird die deviceid 5 mal gesendet, es knnen also 256 IDs eingestellt werden
  #warning "Compiliere fur Wintergarten"
#elif defined WINTERGARTEN_SLAVE
  #warning "Compiliere fur Wintergarten_slave"
  #define DEVICEID  0x60        // Um Speicher zu sparen wird die deviceid 5 mal gesendet, es knnen also 256 IDs eingestellt werden
  #define DEVICEID_LAUSCH  0x58        // Deviceid der anderen Bausteine des Busses auf dem fuer diese nur geluscht wird. 
#else
#error "Keine Zieladresse angegeben, muss WOHNZIMMER; WINTERGARTEN etc.sein"
#endif


#define CRCVAL  0x54   // aus http://www.zorc.breitbandkatze.de/crc.html


   // Mit order = 8, Polynom 0x31, Initial 00, final 0  data: %aa%aa%aa%aa%aa%aa%10
                           // Übertragen wird CRC 6mal DEVICEID FAMILYCODE

#ifdef __AVR_ATtiny13__
#define ONEWIREPIN 1         // Pin, an dem der 1-Wire angeschlossen ist, MUSS INT0 sein
#define PRESENCEZEIT 15      // 120 us bei 8 MHz
#define PRESENCEWAITZEIT 3    // Gendert von 2 auf 3
#define ABTASTZEIT 3           // Abtastzeitpunkt beim empfangen 
#define SENDEZEIT 4           //  Dauer des 0-Sendeimpulses
#define ONEWIRE_INT_ROUTINE (ISR (INT0_vect))

#elif defined (__AVR_ATmega8__)
#define PAD_RELAIS  0        
#define TIMSK0 TIMSK        // TIMSK0 istTIMSK beim Attiny13
#define TIMSK1 TIMSK        // TIMSK0 istTIMSK beim Attiny13
#define TIFR1 TIFR
#define EEPE EEWE
#define EEMPE EEMWE

#define ONEWIREPIN 2        // Pin, an dem der 1-Wire angeschlossen ist, MUSS INT0 sein
#define PRESENCEZEIT 15
#define PRESENCEWAITZEIT 3
#define ABTASTZEIT 2           // War 5 jetzt 3
#define SENDEZEIT 5           // War 15 jetzt auf 10
#define BAUDRATE 38400
#define Onewire_Pin_set_Input (DDRD &= ~(1 << ONEWIREPIN))
#define Onewire_Pin_set_Output (DDRD |= (1 << ONEWIREPIN))
#define Onewire_Pin_check_One (PIND & (1 << ONEWIREPIN))
#define Onewire_Pin_clear  (PORTD &= ~(1 << ONEWIREPIN))
#define ONEWIRE_INT_ROUTINE (ISR (INT0_vect))

#else                            // MEGA169p
#define PAD_RELAIS  0        
//#define TIMSK0 TIMSK        // TIMSK0 istTIMSK beim Attiny13
//#define TIFR0 TIFR
#define EEPE EEWE
#define EEMPE EEMWE

#define ONEWIREPIN 0        // Pin, an dem der 1-Wire angeschlossen ist, MUSS INT0 sein
#define PRESENCEZEIT 15
#define PRESENCEWAITZEIT 3
#define ABTASTZEIT 2           // War 5 jetzt 3
#define SENDEZEIT 5           // War 15 jetzt auf 10
#define BAUDRATE 38400

#define Onewire_Pin_set_Input (DDRE &= ~(1 << ONEWIREPIN))
#define Onewire_Pin_set_Output (DDRE |= (1 << ONEWIREPIN))
#define Onewire_Pin_check_One (PINE & (1 << ONEWIREPIN))
#define Onewire_Pin_check_One_pine (pine & (1 << ONEWIREPIN))
#define Onewire_Pin_clear  (PORTE &= ~(1 << ONEWIREPIN))
#define timer1_need_clock() (TCCR1B & ((1<<CS12)|(1<<CS11)|(1<<CS10)))
#define ONEWIRE_INT_ROUTINE void onewire_ext_int(char pine) 


#endif    // Port als Ausgang

// Onewire Variablen
volatile unsigned char status_global;      // Status of 1-wire statemachine
volatile unsigned char bitcount;         // Counter for received bits within byte
volatile unsigned char transbyte;     // received byte from 1-wire bus
volatile unsigned char bytecount;     // counter for received or transmitted bytes on the 1-wire bus
volatile unsigned char shift_reg;     // shiftregister for CRC Berechnung
volatile unsigned char fb_bit;         // fr CRC Berechnung
volatile unsigned char transdata[9];   // Storage for data to be send or received,
					// similar to 18B20 Tempsensor. Also for die EEPROM Bytes
					// Transdata_bruch has the  fracrions, in 16tel
volatile unsigned char onewire_cmd1; 
volatile unsigned char onewire_cmd2;     // commands received from onewire 
volatile unsigned char counter2val;         // erstmal auf 0 zur Einschaltverzoegerung
volatile unsigned char counter2val_pre;         // erstmal auf 0 zur Einschaltverzoegerung
volatile unsigned char lauschmode;         // Onewire on slavemode
volatile signed char temperatur[8];                // Temperatur for up to 8 values /entsprechend 8 ADC
volatile signed char temperatur_bruch[8];                // Temperaturfractions for up to 8 values /entsprechend 8 ADC
volatile signed char temperatur_read[10];           // Temperaturen, die von anderen devices gesendet wurden und die mitgehrt wurden
//volatile signed char scratchpad_read[2];           // Scratchpaddaten, die von anderen devices gesendet wurden und die mitgehrt wurden

